Overview
========

This is a repo of just the modified bits of a uClinux tree needed to
get libreswan building.

To make it usable go to lantronix and get their linux SDK and layer
this on top of that.  How?  Suggestions welcome :-)

For building instructions see the next session.

For what's left see: TODO.

For how to hack in this git repo see: Workflow.

For a summary of issues addressed see: What was "fixed".


Building Libreswan (IPSEC) for Lantronix XPort Pro (m68k)
=========================================================


Preparation
-----------

Three things are required:


#. The official SDK from Lantronix

   This is available from their web site; look for the information in
   the forum pages.


#. A more up-to-date compiler

   See: http://www.uclinux.org/pub/uClinux/m68k-elf-tools/

   Release 20101118 was used when making these notes.

   The problem is that NSS is so big it causes the default 64k GOT
   (global offset table) to overflow, and the workaround (compile with
   -mxgot) doesn't work with the official toolchain (elf2flt doesn't
   handle the 32-bit relocations that -mxgot creates).


#. this repository of changed files

   This repository just contains changed and added files.  It should
   be overlaid on the official Lantronix SDK.


Next step build:


#. set up your environment

      cd ~/linuxsdk
      . ./env_m68k-uclinux
      make distclean

   and check you're picking up the newer m68k compiler

      which m68k-uclinux-gcc
      /usr/local/bin/m68k-uclinux-gcc


#. create and build a development configuration

   start menu config:

      make menuconfig

   Select the development kernel.  From the main "uClinux Distribution
   Configuration" screen:
      
      Vendor/Product Selection  --->
         Configuration Profile (DEFAULT)  --->
	    (X) DEVELOPMENT 

   Select kernel and application setting configuration.  From the main
   "uClinux Distribution Configuration" screen:

      Kernel/Library/Defaults Selection  --->
         [*] Customize Kernel Settings
         [*] Customize Application/Library Settings

   Then exit out (save when asked).  If you get prompted about "(NEW)"
   items, just hit return for now.

   You'll find yourself in...


#. Customise the kernel configuration options:

   Enable XFRM et.al.  From the main "Linux Kernel Configuration"
   screen:

      [*] Networking support  --->
         Networking options  --->

   and then ensure that the following are selected:

	    [*] PF_KEY sockets
	    [*] Transformation user configuration interface
                (only appears when PF_KEY sockets is enabled)

	    [*]   IP: advanced router 

	    [*]   IP: ESP transformation
	    [*]   IP: IPComp transformation (to force INET_XFRM_TUNNEL)

	    [*]   IP: IPsec tunnel mode 

   Enable some extra ciphers.  From the main "Linux Kernel
   Configuration" screen:

      -*- Cryptographic API  --->
          [*]   GCM/GMAC support 
          -*-   CBC support
	  [*]   CTR support
	  -*-   HMAC support
	  -*-   MD5 digest algorithm
	  -*-   SHA1 digest algorithm	  
	  [*]   AES cipher algorithms


#. Customise the userland configuration options (pass one):

   Enable libreswan.  From the main "uClinux Distribution
   Configuration" screen:

      Network Applications  --->
        [*] libreswan-apps

   Increase the size of the busybox (shell) input buffer so that it
   doesn't overflow.  From the main "uClinux Distribution
   Configuration" screen:

      BusyBox  ---> 
         Busybox Settings  --->
	    Busybox Library Tuning  --->
	       (2048) Maximum length of input

   
    Consider adding some additional utilities that aid aid with
    debugging.  From the main "uClinux Distribution Configuration"
    screen:

      Network Applications  --->
	[*] iproute2
	    [*]   ip
        [*] tcpdump

   Check that the required libraries were enabled.  From the main
   "uClinux Distribution Configuration" screen, check for:

      Library Configuration  --->
         --- Build libGMP
         --- Build NSS+NSPR

   Then exit out, and save.


#. enable GLOB_NOMAGIC in uClib's libgmp configuration

   There's a problem here.  The uClib configuration file needs
   changing but that only seems to be possible after make has been run
   at least once.  To get around this, a build that is expected to
   fail is first run, vis:

      make
      Error: .....

   Now that linux/lib/uClibc/.config has been created it can be
   tweaked.

   Either, in linux/lib/uClibc/.config (it was created by the above)
   find the line:

      # UCLIBC_HAS_GNU_GLOB is not set

   and change it to:

      UCLIBC_HAS_GNU_GLOB=y

   Or, "cd" to linix/lib/uClibc and then run "make menuconfig";
   GLOB_NOMAGIC is under "Big and Tall".

   Better fix welcome.


#. build

      make


#. get everything set up in the NFS root directory

   #. copy the "whack" and "pluto" binaries to the board's NFS root
      directory (TODO: need to get the build cleaned up).

         cp linux/user/libreswan/build/libreswan-3.12/OBJ.uclinux-dist/programs/pluto/{whack,pluto} linux/nfs/

   #. unpack etc-ipsec-d.tar.bz2 into the NFS root directory (TODO:
      get certutil working)

         bunzip2 < etc-ipsec-d.tar.bz2 | ( cd linux/nfs && sudo tar xpvf - )

   #. create /etc/ipsec.conf containing just:

         config setup
            protostack=netkey


#. On the board:

   #. start pluto (no arguments); logs in /var/log/messages:

         ./pluto --debug-all &

   #. try whack


TODO
====

High priority
-------------

Determine if that huge, but commented out, function found in SQLITE is
actually used by NSS.

Correctly install the NSS libraries (and binaries?).  Requires changes
to linux/lib/Makefile? so that they get copied to the correct
location.  This will let us simplify the pluto link line.

Cleanup libreswan's include and library paths.  Needs NSS libraries to
be installed correctly.

Install libreswan binaries in the correct place.

Determine why "certutil -d sql:/etc/ipsec.d -N" crashes (it tries to
allocate a 2.5M chunk of memory; why?).

Update NSS to current?

Update libreswan to current?  This may require an update to NSS; and
will require an import of libevent.

Second attempt to run pluto fails (trying to allocate 2.5M).  Suspect
NSS as certutil fails in a similar way on its first attempt.

Low priority
------------

Set up libreswan's "make menuconfig" to be more like "openswan".  That
is, with the ability to select the programs that are built and
installed.

Set up NSS's "make menuconfig" to be more like "openswan".  That is,
with the ability to select the programs that are built and installed.

Get more of the NSS binaries running; some just required libz.  Since
the needed commands "certutil" and "pk12util" build this isn't
critical.


What was "fixed"
================

For reference, this section notes each of the problems encountered.
If you're looking to forward port then this list of issues might be a
good start.


NSS+NSPR
--------

#. Need to build NSS statically

   Mozilla bugs 533014 and 534471 contain patches to help with this.
   They have been partially merged.

   For instance: the patches namespace proof some common function
   names and that has been applied; where as the attempt to avoid -ldl
   and dl*() calls is less complete.

#. NSPR's configure.in has autoconf 2.13 problems

   After autoconf 2.13, the use of target, host, and build changed and
   NSPR's configure.in has never really been properly updated.

   For instance: tests for "$target" != "$host" (should instead check
   "$build"); the compiler variable names like HOST_CC are
   out-of-date.

#. NSS/NSPR think they know the correct compiler flags

   For instance, forcing -m68020-60 and -O and -fPIC.

#. uClinux Makefile's "export all" interacts badly with NSS+NSPR

   For instance, instead of using local DIR it uses one defined higher
   up.  May need to add lots of unexport lines.

#. fork() calls

   For instance, nss/lib/freebl/unix_rand.c

#. very large functions

   For instance, nss/lib/sqlite/sqlite3.c:sqlite3VdbeExec

#. linux audit assumed when LINUX

   For instance, nss/lib/softoken/fipstokn.c


Libreswan
---------

#. lots of stuff needed turning off, for instance ADNS

#. the existing import assumes fork and other stuff

#. pluto contains calls to fork() even when no-fork

#. need to explicitly specify all the archives

#. klips doesn't fully disable when disabled

#. wants to fork addconn

#. many programs don't need to be built


Hacking Workflow
================

First, you'll likely want to set the following GIT options:

    git config status.showUntrackedFiles no

which will stop git complaining about the files not yet in the
repository.


If you need to modify a file that is part of the uCLinux distro:


#. if not already, add it, unmodified to the GIT repo; use the commit
   message "import"

#. modify the file

#. commit the changes


If you need to patch either NSS or libreswan source FILE, then you'll
need to add a "z" series patch to {lib/nss,user/libreswan}/patches:


#. go to the relevant build directory:

      cd linux/lib/nss/build

   or:

      cd linux/user/libreswan/build

#. save a copy of the original file

      cp ${FILE}{,.LSW}

#. edit ${FILE}

#. create a patch and save it in ../patches; all the local patches are
   ordered as z01, z02, ...

      diff -u ${FILE}{.LSW,} > ../patches/<tab>-zNN-......patch
